{

open Parser

let builtins = ["print"; "cons"; "nil"; "null"; "head"; "tail"]

let inc_lineno lexbuf =
  let pos = lexbuf.Lexing.lex_curr_p in
    lexbuf.Lexing.lex_curr_p <- { pos with
                                    Lexing.pos_lnum = pos.Lexing.pos_lnum + 1;
                                    Lexing.pos_bol = pos.Lexing.pos_cnum;
                                }

let inc_lineno_by_string lexbuf s =
  let inc = function
      '\n' -> inc_lineno lexbuf
    | _    -> () 
  in
    String.iter inc s

let cur_token = ref None

let return_token tok = cur_token := Some tok; tok

let string_of_token = function
    LET -> "LET"
  | IN -> "IN"
  | FUN -> "FUN"
  | IF -> "IF"
  | THEN -> "THEN"
  | ELSE -> "ELSE"
  | CALL_CC -> "CALL_CC"
  | EQUALS -> "EQUALS"
  | EEQUALS -> "EEQUALS"
  | PLUS -> "PLUS"
  | MULT -> "MULT"
  | LPAREN -> "LPAREN"
  | RPAREN -> "RPAREN"
  | COMMA -> "COMMA"
  | IDENT(x) -> "IDENT(" ^ x ^ ")"
  | BUILTIN(x) -> "BUILTIN(" ^ x ^ ")"
  | INT(i) -> "INT(" ^ string_of_int i ^ ")"
  | EOF -> "EOF"

let error_msg lexbuf = 
  let string_of_position p = 
    p.Lexing.pos_fname ^ ":" ^ string_of_int p.Lexing.pos_lnum ^ ":" 
    ^ string_of_int (p.Lexing.pos_cnum - p.Lexing.pos_bol)
  in
    string_of_position lexbuf.Lexing.lex_curr_p ^ ": parse error" ^
      (match !cur_token with
           None -> ""
         | Some tok -> " (next token: " ^ string_of_token tok ^ ")")
}

let dec_digit = ['0' - '9']
let sign = '-' | ""
let int_lit = sign dec_digit+

let letter = ['a' - 'z' 'A' - 'Z']
let ident_start = letter | '_'
let ident_rest = ident_start | dec_digit
let ident = ident_start ident_rest*

rule token = parse
    [' ' '\t' '\r']+        { token lexbuf }
  | ['\n']                 { inc_lineno lexbuf; token lexbuf }
  | "let"                   { return_token LET }
  | "in"                    { return_token IN }
  | "fun"                   { return_token FUN }
  | "if"                    { return_token IF }
  | "then"                  { return_token THEN }
  | "else"                  { return_token ELSE }
  | "call/cc"               { return_token CALL_CC }
  | '='                     { return_token EQUALS }
  | "=="                    { return_token EEQUALS }
  | '+'                     { return_token PLUS }
  | '*'                     { return_token MULT }
  | '('                     { return_token LPAREN }
  | ')'                     { return_token RPAREN }
  | ','                     { return_token COMMA }
  | ident as lxm            { if List.mem lxm builtins 
                              then return_token (BUILTIN lxm)
                              else return_token (IDENT lxm) }
  | int_lit as lxm          { return_token (INT (int_of_string lxm)) }
  | eof                     { return_token EOF }

