%token LET IN FUN IF THEN ELSE CALL_CC
%token EQUALS EEQUALS PLUS MULT LPAREN RPAREN COMMA
%token <string> IDENT, BUILTIN
%token <int> INT
%token EOF

%start exp
%type <string Syntax.t> exp

%%

exp: INT                             { Syntax.Int ($1) }
   | exp PLUS exp                    { Syntax.Builtin ("+", [$1; $3]) }
   | exp MULT exp                    { Syntax.Builtin ("*", [$1; $3]) }
   | exp EEQUALS exp                 { Syntax.Builtin ("==", [$1; $3]) }
   | CALL_CC LPAREN exp RPAREN       { Syntax.Builtin ("call/cc", [$3]) }
   | BUILTIN LPAREN exp_list RPAREN  { Syntax.Builtin ($1, $3) }
   | BUILTIN                         { Syntax.Builtin ($1, []) }
   | IF exp THEN exp ELSE exp        { Syntax.If ($2, $4, $6) }
   | IDENT                           { Syntax.Ident ($1) }
   | LET IDENT EQUALS exp IN exp     { Syntax.Let ($2, $4, $6) }
   | FUN IDENT LPAREN id_list RPAREN 
       EQUALS exp                    { Syntax.Function ($2, $4, $7) }
   | exp LPAREN exp_list RPAREN      { Syntax.Apply ($1, $3) }
   | LPAREN exp RPAREN               { $2 }
   ;

exp_list: r_exp_list  { List.rev $1 };
r_exp_list: exp                   { [$1] }
          | r_exp_list COMMA exp  { $3 :: $1 }
          ;

id_list: r_id_list  { List.rev $1 };
r_id_list: IDENT                  { [$1] }
         | r_id_list COMMA IDENT  { $3 :: $1 }
         ;
